@@ -470,6 +470,10 @@ static int starter_whack_basic_add_conn(struct starter_config *cfg
 		}
 	}
 
+	if(conn->options_set[KBF_FORCEENCAP]) {
+		msg.forceencaps=conn->options[KBF_FORCEENCAP];
+	}
+
 	set_whack_end(cfg, "left",  &msg.left, &conn->left);
 	set_whack_end(cfg, "right", &msg.right, &conn->right);
 
