@@ -11,7 +11,6 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
  *
- * RCSID $Id: constants.c,v 1.14 2005/09/27 02:19:05 mcr Exp $
  */
 
 /*
@@ -921,9 +920,9 @@ enum_names ppk_names = { PPK_PSK, PPK_XAUTH, ppk_name, NULL };
 const char *const natt_type_bitnames[] = {
   "draft-ietf-ipsec-nat-t-ike-00/01",    /* 0 */
   "draft-ietf-ipsec-nat-t-ike-02/03",
-  "draft-ietf-ipsec-nat-t-ike (OS X)",
-  "RFC 3947 (NAT-Traversal)",
-  "4",   "5",   "6",   "7",              /* 4 */ 
+  "draft-ietf-ipsec-nat-t-ike (MacOS X)", 
+  "RFC 3947 (NAT-Traversal)",		/* 3 */
+  "4",   "5",   "6",   "7", 
   "8",   "9",   "10",  "11",
   "12",  "13",  "14",  "15",
   "16",  "17",  "18",  "19", 
