@@ -115,7 +115,6 @@ usage(const char *mess)
 	    " \\\n\t"
 	    "[--nofork]"
 	    " [--stderrlog]"
-	    " [--use-nostack]"         /* old --no_klips */
 	    " [--nocrsend]"
 	    " [--strictcrlpolicy]"
 	    " [--crlcheckinterval]"
@@ -124,6 +123,7 @@ usage(const char *mess)
 	    " [--use-auto]"
 	    " [--use-klips]"
 	    " [--use-netkey]"
+	    " [--use-nostack]"         /* old --no_klips */
 	    " \\\n\t"
 	    "[--interface <ifname|ifaddr>]"
 	    " [--ikeport <port-number>]"
@@ -132,6 +132,8 @@ usage(const char *mess)
 	    " \\\n\t"
 	    "[--perpeerlogbase <path>] [--perpeerlog]"
 	    " \\\n\t"
+	    "[--coredir <dirname>] [--noretransmits]"
+	    " \\\n\t"
 	    "[--secretsfile <secrets-file>]"
 	    " [--ipsecdir <ipsec-dir>]"
 	    " \\\n\t"
@@ -148,9 +150,12 @@ usage(const char *mess)
 	    " [--debug-emitting]"
 	    " \\\n\t"
 	    "[--debug-control]"
+	    "[--debug-lifecycle]"
 	    " [--debug-klips]"
 	    " [--debug-x509]"
 	    " [--debug-dns]"
+	    " [--debug-oppo]"
+	    " [--debug-oppoinfo]"
 	    " [--debug-dpd]"
 	    " [ --debug-private]"
 	    " [ --debug-pfkey]"
@@ -367,6 +372,7 @@ main(int argc, char **argv)
 	    { "debug-klips", no_argument, NULL, DBG_KLIPS + DBG_OFFSET },
 	    { "debug-dns", no_argument, NULL, DBG_DNS + DBG_OFFSET },
 	    { "debug-oppo", no_argument, NULL, DBG_OPPO + DBG_OFFSET },
+	    { "debug-oppoinfo", no_argument, NULL, DBG_OPPOINFO + DBG_OFFSET },
 	    { "debug-controlmore", no_argument, NULL, DBG_CONTROLMORE + DBG_OFFSET },
 	    { "debug-dpd", no_argument, NULL, DBG_DPD + DBG_OFFSET },
             { "debug-x509", no_argument, NULL, DBG_X509 + DBG_OFFSET },
